from pycompss.api.task import task
from biobb_common.tools import file_utils as fu
from mmb_api import pdb_variants

@task(output_mutations_list_txt=FILE_OUT)
def pdb_variants_pc(output_mutations_list_txt, properties, **kwargs):
    try:
        pdb_variants.MmbPdbVariants(output_mutations_list_txt=output_mutations_list_txt, properties=properties, **kwargs).launch()
    except Exception:
        traceback.print_exc()
        fu.write_failed_output(output_mutations_list_txt)



#!/usr/bin/env python
import argparse
import requests
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
import re
from biobb_io.mmb_api.common import get_uniprot
from biobb_io.mmb_api.common import get_variants
import logging
logging.getLogger("requests").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)

class MmbPdbVariants(object):
    """Wrapper class for the MMB group UNIPROT REST API.
    This class is a wrapper for the UNIPROT (http://www.uniprot.org/)
    mirror of the MMB group REST API (http://mmb.irbbarcelona.org/api/)

    Args:
        output_mutations_list_txt (str): Path to the TXT file containing an ASCII comma separated values of the mutations.
        properties (dic):
            | - **pdb_code** (*str*): RSCB PDB four letter code. ie: "2ki5".
            | - **url** (*str*): ("http://mmb.irbbarcelona.org/api") URL of the MMB REST API.
    """
    def __init__(self, output_mutations_list_txt, properties, **kwargs):
        self.output_mutations_list_txt = output_mutations_list_txt
        self.pdb_code = properties.get('pdb_code')
        if not self.pdb_code:
            raise ValueError('pdb_code property undefined or empty')
        self.url = properties.get('url',"http://mmb.irbbarcelona.org/api")
        self.global_log= properties.get('global_log', None)
        self.prefix = properties.get('prefix',None)
        self.step = properties.get('step',None)
        self.path = properties.get('path','')

    def launch(self):
        """Writes the variants of the selected
        `self.pdb_code` to `self.output_mutations_list_txt`
        """
        out_log, _ = fu.get_logs(path=self.path, prefix=self.prefix, step=self.step)
        uniprot_id = get_uniprot(self.pdb_code, self.url, out_log, self.global_log)
        url_mapPDBRes = (self.url+"/uniprot/"+uniprot_id+"/mapPDBRes?pdbId="+self.pdb_code)
        pattern = re.compile(("p.(?P<wt>[a-zA-Z]{3})(?P<resnum>\d+)(?P<mt>[a-zA-Z]{3})"))

        out_log.info('Fetching variants for uniprot_id: '+uniprot_id+' and pdb_code: '+self.pdb_code)
        if self.global_log:
            self.global_log.info(fu.get_logs_prefix()+'Fetching variants for uniprot_id: '+uniprot_id+' and pdb_code: '+self.pdb_code)

        unfiltered_dic = requests.get(url_mapPDBRes).json()
        if not unfiltered_dic: return []

        mapdic = requests.get(url_mapPDBRes).json()
        mutations = []
        uniprot_var_list = get_variants(uniprot_id, self.url, out_log, self.global_log)
        for var in uniprot_var_list:
            uni_mut = pattern.match(var).groupdict()
            for k in mapdic.keys():
                for fragment in mapdic[k]:
                    if int(fragment['unp_start']) <= int(uni_mut['resnum']) <= int(fragment['unp_end']):
                        resnum = int(uni_mut['resnum']) + int(fragment['pdb_start']) - int(fragment['unp_start'])
                        mutations.append(k[-1]+'.'+uni_mut['wt']+str(resnum)+uni_mut['mt'])

        out_log.info('Found '+str(len(mutations))+' mutations mapped to PDB: '+self.pdb_code)
        out_log.info(str(mutations))
        out_log.info('Writting mutations to: '+self.output_mutations_list_txt)
        if self.global_log:
            self.global_log.info(fu.get_logs_prefix()+'Found '+str(len(mutations))+' mutations mapped to PDB: ' + self.pdb_code)
            self.global_log.info(fu.get_logs_prefix()+str(mutations))
            self.global_log.info(fu.get_logs_prefix()+'Writting mutations to: '+self.output_mutations_list_txt)

        with open(self.output_mutations_list_txt, 'w') as mut_file:
            mut_file.write(",".join(mutations))

def main():
    parser = argparse.ArgumentParser(description="Wrapper for the PDB Variants (http://www.rcsb.org/pdb/home/home.do) mirror of the MMB group REST API (http://mmb.irbbarcelona.org/api/)")
    parser.add_argument('--conf_file', required=True)
    parser.add_argument('--system', required=True)
    parser.add_argument('--step', required=True)

    #Specific args of each building block
    parser.add_argument('--output_mutations_list_txt', required=True)
    ####

    args = parser.parse_args()
    properties = settings.YamlReader(conf_file_path=args.conf_file, system=args.system).get_prop_dic()[args.step]

    #Specific call of each building block
    MmbPdbVariants(output_mutations_list_txt=args.output_mutations_list_txt, properties=properties).launch()
    ####

if __name__ == '__main__':
    main()



import requests
import json
from biobb_common.tools import file_utils as fu
import logging
logging.getLogger("requests").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)


def download_pdb(pdb_code, url="http://mmb.irbbarcelona.org/api/pdb/", filt='filter=/1&group=ATOM', out_log=None, global_log=None):
    """
    Returns:
        String: Content of the pdb file.
    """
    url = url+pdb_code.lower()+"/coords/?"+filt

    if out_log:
        out_log.info("\nDownloading:\npdb_code: "+pdb_code+"\nfrom: "+url)
    if global_log:
        global_log.info(fu.get_logs_prefix()+"Downloading: "+pdb_code+"from: "+url)

    return requests.get(url).content.decode('utf-8')

def get_cluster_pdb_codes(pdb_code, url="http://mmb.irbbarcelona.org/api/pdb/", cluster='90', out_log=None, global_log=None):
    """
    Returns:
        String list: The list of pdb_codes of the selected cluster.
    """
    pdb_codes = set()

    url = url+pdb_code.lower()+'/clusters/cl-'+cluster+".json"
    cluster_list = json.loads(requests.get(url).content.decode('utf-8'))['clusterMembers']
    for elem in cluster_list:
        pdb_codes.add(elem['_id'].lower())

    if out_log:
        out_log.info('Cluster: '+cluster+' of pdb_code: '+pdb_code+'\n List: '+str(pdb_codes))
    if global_log:
        global_log.info(fu.get_logs_prefix()+'Cluster: '+cluster+' of pdb_code: '+pdb_code+'\n List: '+str(pdb_codes))

    return pdb_codes

def get_uniprot(pdb_code, url="http://mmb.irbbarcelona.org/api", out_log=None, global_log=None):
    """Returns the UNIPROT code corresponding to the `pdb_code`.

    Returns:
        str: UNIPROT code.
    """
    url_uniprot_id = (url+"/pdb/"+pdb_code.lower()+"/entry/uniprotRefs/_id")
    uniprot_id = requests.get(url_uniprot_id).json()['uniprotRefs._id'][0]

    if out_log:
        out_log.info('PDB code: '+pdb_code+' correspond to uniprot id: '+uniprot_id)
    if global_log:
        global_log.info('PDB code: '+pdb_code+' correspond to uniprot id: '+uniprot_id)

    return uniprot_id

def get_variants(uniprot_id, url="http://mmb.irbbarcelona.org/api", out_log=None, global_log=None):
    """Returns the variants of the `uniprot_id` code.

    Returns:
        :obj:`list` of :obj:`str`: List of variants.
    """
    url_uniprot_mut = (url+"/uniprot/"+uniprot_id+"/entry/variants/vardata/mut/?varorig=humsavar")
    variants = requests.get(url_uniprot_mut).json()['variants.vardata.mut']
    variants = variants if variants else []

    if out_log:
        out_log.info('Found: '+str(len(variants))+' variants for uniprot id: '+uniprot_id)
        out_log.info(str(variants))
    if global_log:
        global_log.info('Found: '+str(len(variants))+' variants for uniprot id: '+uniprot_id)
        global_log.info(str(variants))

    return variants if variants else []



from pycompss.api.task import task
from biobb_common.tools import file_utils as fu
from mmb_api import pdb_cluster_zip

@task(output_pdb_zip_path=FILE_OUT)
def pdb_cluster_zip_pc(output_pdb_zip_path, properties, **kwargs):
    try:
        pdb_cluster_zip.MmbPdbClusterZip(output_pdb_zip_path=output_pdb_zip_path, properties=properties, **kwargs).launch()
    except Exception:
        traceback.print_exc()
        fu.write_failed_output(output_pdb_zip_path)



#!/usr/bin/env python

import argparse
import os
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
import logging
from biobb_io.mmb_api.common import download_pdb
logging.getLogger("requests").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)

class MmbPdb(object):
    """Wrapper class for the MMB group PDB REST API.
    This class is a wrapper for the PDB (http://www.rcsb.org/pdb/home/home.do)
    mirror of the MMB group REST API (http://mmb.irbbarcelona.org/api/)

    Args:
        output_pdb_path (str): Path to the output PDB file.
        properties (dic):
            | - **pdb_code** (*str*) - RSCB PDB code. ie: "2VGB"
            | - **filter** (*str*) - ("filter=/1&group=ATOM") Filter for the :meth:`biobb_io.mmb_api.MmbPdb.get_pdb_zip` method following the J(s)Mol format.
            | - **url** (*str*) - ("http://mmb.irbbarcelona.org/api/pdb/") URL of the MMB PDB REST API.

    """
    def __init__(self, output_pdb_path, properties, **kwargs):
        self.output_pdb_path = output_pdb_path
        self.url = properties.get('url',"http://mmb.irbbarcelona.org/api/pdb/")
        self.pdb_code = properties.get('pdb_code').strip().lower()
        self.filt = properties.get('filter', 'filter=/1&group=ATOM')
        self.global_log= properties.get('global_log', None)
        self.prefix = properties.get('prefix',None)
        self.step = properties.get('step',None)
        self.path = properties.get('path','')

    def launch(self):
        """
        Writes the PDB file content of the first pdb_code
        to output_pdb_path.
        """
        out_log, _ = fu.get_logs(path=self.path, prefix=self.prefix, step=self.step)

        #Downloading PDB_files
        pdb_string = download_pdb(self.pdb_code, self.url, self.filt, out_log, self.global_log)

        out_log.info("\nWritting: "+self.pdb_code+"\nto: "+os.path.abspath(self.output_pdb_path))
        if self.global_log:
            self.global_log.info(fu.get_logs_prefix()+"Writting: "+self.pdb_code+"\nto: "+os.path.abspath(self.output_pdb_path))

        with open(self.output_pdb_path, 'w') as f:
            f.write(pdb_string)

def main():
    parser = argparse.ArgumentParser(description="Wrapper for the PDB (http://www.rcsb.org/pdb/home/home.do) mirror of the MMB group REST API (http://mmb.irbbarcelona.org/api/)")
    parser.add_argument('--conf_file', required=True)
    parser.add_argument('--system', required=True)
    parser.add_argument('--step', required=True)

    #Specific args of each building block
    parser.add_argument('--output_pdb_path', required=True)
    ####

    args = parser.parse_args()
    properties = settings.YamlReader(conf_file_path=args.conf_file, system=args.system).get_prop_dic()[args.step]

    #Specific call of each building block
    MmbPdb(output_pdb_path=args.output_pdb_path, properties=properties).launch()
    ####

if __name__ == '__main__':
    main()
